angular
	.module( "beamng.apps" )
	.directive( "evBattery", [ "StreamsManager", function ( StreamsManager ) {
		return {
			template:
				"<object class='bngApp' style='width:100%; height:100%; pointer-events: none' type='image/svg+xml' data='modules/apps/EvBatteryMonitor/ev-battery-monitor.svg?t=" + Date.now() + "'/>",
			replace: true,
			restrict: "EA",
			link: function ( scope, element, attrs ) {
				function clamp( val, min, max ) {
					return Math.max( Math.min( val, max ), min );
				}

				StreamsManager.add( [ "hybrid" ] );

				scope.$on( "$destroy", function () {
					StreamsManager.remove( [ "hybrid" ] );
				} );

				element.on( "load", function () {
					var svg = element[ 0 ].contentDocument;
					var soc = {
						bar: svg.getElementById( "fill-bat" ),
						val: svg.getElementById( "val-bat" ),
						height: svg.getElementById( "fill-bat" ).getAttribute( "height" ),
						factor: svg.getElementById( "fill-bat" ).getAttribute( "height" ) / 100.0,
						y: +svg.getElementById( "fill-bat" ).getAttribute( "y" )
					};
					var kw = {
						val: svg.getElementById( "val-kw" ),
						pos: {
							bar: svg.getElementById( "fill-kw-pos" ),
							height: svg.getElementById( "fill-kw-pos" ).getAttribute( "height" ),
							factor: svg.getElementById( "fill-kw-pos" ).getAttribute( "height" ) / 2.0,
							y: +svg.getElementById( "fill-kw-pos" ).getAttribute( "y" )
						},
						neg: {
							bar: svg.getElementById( "fill-kw-neg" ),
							factor: svg.getElementById( "fill-kw-neg" ).getAttribute( "height" ) / 2.0,
						}
					}

					scope.$on( "streamsUpdate", function ( event, streams ) {
						if ( streams != null && streams.hybrid != null ) {
							var socVal = Math.round( streams.hybrid.soc * 100 );
							var kwVal = Math.round( streams.hybrid.kw );
							var maxKw = streams.hybrid.maxKw;

							soc.val.innerHTML = `${socVal}%`;
							kw.val.innerHTML = `${kwVal} kW`;

							var socHeight = isNaN( socVal ) ? 0 : socVal * soc.factor;
							soc.bar.setAttribute( "height", socHeight );
							soc.bar.setAttribute( "y", soc.y + ( soc.height - socHeight ) );

							var kwPosHeight = 0;
							var kwNegHeight = 0;

							if ( maxKw === 0 ) {
								kwPosHeight = kwNegHeight = 0;
							} else {
								var kwPosNormalized = isNaN( kwVal ) ? 0 : clamp( kwVal / maxKw, 0, 1 );
								var kwNegNormalized = isNaN( kwVal ) ? 0 : clamp( kwVal / maxKw, -1, 0 );

								kwPosHeight = Math.abs( kwPosNormalized ) * kw.pos.factor;
								kwNegHeight = Math.abs( kwNegNormalized ) * kw.neg.factor;
							}

							kw.pos.bar.setAttribute( "height", kwPosHeight );
							kw.pos.bar.setAttribute( "y", kw.pos.y + (kw.pos.height - kwPosHeight) );

							kw.neg.bar.setAttribute( "height", kwNegHeight );
						}
					} );
				} );
			}
		};
	} ] );
